<?php
/*
 * Copyright (c) 2009, The members of Ouga! project
 * Etienne Deparis <etienne.deparis@umaneti.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *
 * modification, are permitted provided that the following conditions
 * are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the Ouga! project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * Access Control List system
 */

class acl {

	private $logs;
	private $cache;

	public function __construct(){
		$this->logs = array();
		$this->cache = array();
	}

	public function getLogs(){
		return $this->logs;
	}

	public function check($user, $path){

		if(is_array($path)){
			$tmp = '';
			$i = 0;
			foreach($path as $p){
				if(($i > 1 || $p == '<none>' || ctype_digit($p))
					&& !in_array(
						$p,
						array('ok', 'nok', 'add', 'remove')
					)
				){
					$p = '#';
				}
				$tmp .= $p . '/';
				$i++;
			}
			$path = substr($tmp, 0, -1);
		}

		if(isset($this->cache[$user . '@' . $path])){
			$this->logs['acl_logs'][]
				= 'ACL test: already in cache';
			return $this->cache[$user . '@' . $path];
		}

		$this->cache[$user . '@' . $path] = false;

		if($user != null
			&& $user != ''
			&& $user != 'Anonymous'){

			$u_aid = bdd_query(
				"select aid from {acl_users} where login = '$user'"
			);
			$d_u_aid = bdd_fetch($u_aid);
			$user_aid = $d_u_aid['aid'];
		}else $user_aid = 0;

		$logs['acl_user_level'] = $user_aid;

		$p_aid = bdd_query(
			"select aid from {acl_paths} where path = '$path'"
		);
		$d_p_aid = bdd_fetch($p_aid);
		$path_aid = $d_p_aid['aid'];

		if($path_aid == '' || $path_aid == null) $path_aid = 0;

		$this->logs['acl_logs'][]
			= 'ACL test: user level = '
			. $user_aid . ' against path (' . $path . ') level = '
			. $path_aid;

		if(!is_numeric($path_aid)){
			if(in_array($user_aid, unserialize($path_aid))){
				$this->cache[$user . '@' . $path] = true;
			}
		}else{
			if($path_aid == 0){
				$this->cache[$user . '@' . $path] = true;
			}else{
				$this->cache[$user . '@' . $path]
					= ($user_aid == $path_aid);
			}
		}

		return $this->cache[$user . '@' . $path];
	}

	public function init(&$variables){

		$queries = array();
		$queries[] = "insert into {acl_groups}(aid, label) values(1, '"
			. "Basic users')";
		if($data = bdd_query("select login from {users}")){
			while($d = bdd_fetch($data)){
				$queries[] = "insert into {acl_users}(login) values('"
					. $d['login'] . "')";
			}
		}
		bdd_query($queries);

		$acl_pathes = array();

		foreach($variables['modules_list'] as $m){
			$this->logs['acl_logs'][]
				= 'Init install ACL for '.$m;

			if($url = _exists($m.'/install.php')){
				require_once $url;
				$func = $m.'_acl_install';
				if(function_exists($func)){
					$acl_pathes[] = $func();
				}
			}
		}

		if(sizeof($acl_pathes) != 0){
			$query = 'insert into {acl_paths} values';

			foreach($acl_pathes as $m){
				foreach($m['primary'] as $p => $title){
					$query .= "('$p', '0', '', '$title'),";
				}
				foreach($m['copy'] as $p => $copy){
					$query .= "('$p', '0', '$copy', ''),";
				}
			}
			$this->logs['acl_logs'][]
				= 'ACL init query to install previous discovered acl: '
				. $query;
			bdd_query(substr($query,0,-1));
		}
	}

	public function discover_acl(&$variables){

		if(is_array($variables['modules_just_setup'])){
			$acl_pathes = array();
			foreach($variables['modules_just_setup'] as $m){
				$this->logs['acl_logs'][]
					= 'Install ACL for '.$m;

				if($url = _exists($m.'/install.php')){
					require_once $url;
					$func = $m.'_acl_install';
					if(function_exists($func)){
						$acl_pathes[] = $func();
					}
				}
			}

			if(sizeof($acl_pathes) != 0){
				$query = array();

				foreach($acl_pathes as $m){
					$querytmp = '';

					if(is_array($m['primary'])){
						foreach($m['primary'] as $p => $title){
							$querytmp .= "('$p', '0', '', '$title'),";
						}
					}

					if(is_array($m['copy'])){
						foreach($m['copy'] as $p => $copy){
							$querytmp .= "('$p', '0', '$copy', ''),";
						}
					}

					if($querytmp != ''){
						$query[] = 'insert into {acl_paths} values'
							. substr($querytmp,0,-1);

						$this->logs['acl_logs'][]
							= 'ACL query to install previous discovered acl: '
							. $querytmp;
					}
				}

				bdd_query($query);
			}
		}

		if(is_array($variables['modules_just_uninstalled'])){
			$query = array();
			foreach($variables['modules_just_uninstalled'] as $m){
				$this->logs['acl_logs'][]
					= 'Remove ACL for '.$m;
				$querytmp = "delete from {acl_paths} where path like '"
					. $m . "%'";

				$query[] = $querytmp;

				$this->logs['acl_logs'][]
					= 'ACL query to remove previous discovered acl: '
					. $querytmp;
			}

			bdd_query($query);
		}
	}

	public function discover_user($login){
		$query = "insert into {acl_users}(login, aid) values('"
			. $login . "', 1)";
		bdd_query($query);

	}

}

?>
