<?php
/*
 * Copyright (c) 2009, The members of Ouga! project
 * Etienne Deparis <etienne.deparis@umaneti.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *
 * modification, are permitted provided that the following conditions are
 * met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - Neither the name of the Ouga! project nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * Template system
 */

class display{

	private $variables = array();

	public function __construct(){
	}

	private function get_header_tags(){
		$header = $this->variables['header'];

		// we add the standard css file
		array_unshift(
			$header['style']['screen'],
			path_to_theme($this->variables['theme']) . 'css/style.css'
		);

		/*array_unshift(
			$header['style']['print'],
			path_to_theme($this->variables['theme']) . 'css/print.css'
		);*/

		$output = '';
		foreach($header as $tag => $content){
			if($header[$tag] != null && !empty($header[$tag])){
				foreach($content as $key => $url){
					switch($tag){
						case 'meta' :
							$output .= '<meta name="' . $key
								. '" content="' . $url . '"/>' . "\n";
							break;
						case 'style' :
							if(is_array($url)){
								foreach($url as $u){
									$output.= '<link rel="stylesheet" '
										. 'type="text/css" media="'
										. $key . '" href="'
										. $u . '" />' . "\n";
								}
							}else{
								$output .= '<link rel="stylesheet" '
									. 'type="text/css" media="'
									. $key . '" href="' . $url . '" />'
									. "\n";
							}
							break;
						case 'atom' :
							$output .= '<link rel="alternate" '
								. 'type="application/atom+xml" '
								. 'title="Atom" href="' .$url . '" />'
								. "\n";
							break;
						case 'script' :
							$output .= '<script type="text/javascript" '
								. 'src="' . $url . '"></script>' . "\n";
							break;
					}
				}
			}
		}
		return $output;
	}

	private function get_template_file(){

		switch($this->variables['page']){
			case 'join' :
				set_lang('theme');
				if ($url = _exists(
					'themes/' . $this->variables['theme']
						. '/' . $this->variables['basetempname']
						. 'template_' . $this->variables['module']
						. '.php'
					)
				){
					return $url;
				}

				return _exists('themes/' . $this->variables['theme']
					. '/' . $this->variables['basetempname']
					. 'template.php');
			case '<none>' :
				// on commence par regarder dans les themes
				if ($url = _exists(
					'themes/' . $this->variables['theme'] . '/'
						. $this->variables['basetempname']
						. $this->variables['module']
						. '.php')
				){
					set_lang('theme');
					return $url;
				}
				// sinon on utilise le template par défaut
				set_lang('module');
				return path_to_module($this->variables['module'], false)
					. 'views/' . $this->variables['basetempname']
					. $this->variables['module'].'.php';
			default :
				if ($url = _exists(
					'themes/' . $this->variables['theme'] . '/'
						. $this->variables['basetempname']
						. $this->variables['module']
						. '_' . $this->variables['page'] . '.php'
					)
				){
					set_lang('theme');
					return $url;
				}

				set_lang('module');
				return path_to_module($this->variables['module'], false)
					.'views/' . $this->variables['basetempname']
					. $this->variables['page'] . '.php';
		}

		throw_error('fatal', 'No template file found for module...');
	}

	private function get_flash_messages($category = null){
		$messages = array_merge(
			$this->variables['flash_messages'],
			component('acl')->getLogs()
		);
		$output = '';

		if(is_array($messages)){
			$output .= '<ul class="flash_messages">';
			if($category != null){
				if(isset($messages[$category])){
					foreach($messages[$category] as $msg){
						$output .= '<li class="' . strclean($category)
							. '">' . strclean($msg) . '</li>';
					}
				}
			}else{
				foreach($messages as $msg_class => $msg){
					foreach($msg as $m){
						$output .= '<li class="' . strclean($msg_class)
							. '">' . strclean($m) . '</li>';
					}
				}
			}
			$output .= '</ul>';
		}
		return $output;
	}

	public function get_module_menu($menu){
		$output = '';
		foreach($menu as $key => $value){
			$output .= '<li><a href="' . base_path() . $key . '">'
				. $value . '</a></li>';
		}
		return $output;
	}

	private function dump_error(){
		if(OUGA_ERROR_LEVEL >= 3){
			ob_start();
			echo '<h3>Variables</h3>';
			var_dump($this->variables);
			echo '<h3>$_REQUEST</h3>';
			var_dump($_REQUEST);
			$dumps = ob_get_contents();
			ob_end_clean();
			return $this->get_flash_messages() . '<br/>'
				. $dumps;
		}
	}

	public function render($vars){

		$this->variables = $vars;

		switch($this->variables['controller']){
			case 2 : $this->variables['basetempname'] = 'mob_';break;
			case 1 :
			default : $this->variables['basetempname'] = '';
		}

		// We add the correct lang ressources
		add_lang_path(
			array(
				array(
					_exists('themes/' . OUGA_THEME . '/lang'),
					'theme'
				),
				array(
					_exists('modules/' . $this->variables['module']
						. '/lang'),
					'module'
				),
			)
		);

		$must_end_now = false;

		// on récupère les variables
		extract($this->variables, EXTR_SKIP);

		if(!isset($this->variables['content'])){
			ob_start();
			require_once $this->get_template_file();
			$this->variables['content'] = ob_get_contents();
			ob_end_clean();
		}

		if(!$must_end_now){

			// last hook call to  modify module display
			run_hook(4, $this->variables);

			// On affiche la page générale
			$this->variables['page'] = 'join';
			$content = $this->get_flash_messages('warning')
				. $this->get_flash_messages('error')
				. $this->variables['content'];
			ob_start();
			require_once $this->get_template_file();
			$page = ob_get_contents();
			ob_end_clean();

		}else{
			$page = $this->variables['content'];
		}

		print $page;
	}
}
?>
