<?php
/*
 * Copyright (c) 2009, The members of Ouga! project
 * Maxime Robache <maxime.robache@tremplin-utc.fr>
 *
 * Contributors:
 * Jean-Baptiste Drucker <jbdrucker@gmail.com>
 * Etienne Deparis <etienne.deparis@umaneti.net>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *
 * modification, are permitted provided that the following conditions are
 * met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - Neither the name of the Ouga! project nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * Ldap connexion related function.
 */

/**
 * Used to compare ldap hashes.
 */
function compare_hash($ldaphash, $entry){

	$auth = FALSE;
	if (preg_match ("/{CRYPT}/i", $ldaphash)) {
		$hash = substr($ldaphash, 19);
		$salt = substr($ldaphash, 7, 12);
		$check = crypt($entry, $salt);
		if (strcmp(trim($check),trim($salt.$hash)) == 0)
			$auth = TRUE;
	} else if (preg_match ("/{SSHA}/i", $ldaphash)) {
		$ldaphash = base64_decode(substr($ldaphash, 6));
		$original_hash = substr($ldaphash, 0, 20);
		$salt = substr($ldaphash, 20);
		$new_hash = pack("H*", sha1($entry.$salt));
		if (strcmp($original_hash, $new_hash) == 0)
			$auth = TRUE;
	}
	return $auth;
}

function ldap_connect_user($user, $passwd){

	$ldapconn = ldap_connect(
		OUGA_LDAP_HOSTNAME,
		OUGA_LDAP_PORT
	) or die("Error while connecting to LDAP server.");

	ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);

	$ldapbind = ldap_bind(
		$ldapconn,
		OUGA_LDAP_USERNAME,
		OUGA_LDAP_PASSWORD
	) or die("LDAP bind failed : " . ldap_error($ldapconn));

	$ldapres = ldap_search(
		$ldapconn,
		OUGA_LDAP_ROOT,
		'(uid=' . $user . ')'
	);

	$entries = ldap_get_entries($ldapconn, $ldapres);
	//echo '<pre>';
	//print_r($entries);
	//echo '</pre>';
	//echo $entries[0][$_config_asso[OUGA_LDAP_PASSENTRY]][0];
	$auth = compare_hash($entries[0][OUGA_LDAP_PASSENTRY][0], $passwd);

	ldap_close($ldapconn);

	return $auth;

}

?>
